/*
 * Copyright 2001-2004 Unicode, Inc.
 *
 * Disclaimer
 *
 * This source code is provided as is by Unicode, Inc. No claims are
 * made as to fitness for any particular purpose. No warranties of any
 * kind are expressed or implied. The recipient agrees to determine
 * applicability of information provided. If this file has been
 * purchased on magnetic or optical media from Unicode, Inc., the
 * sole remedy for any claim will be exchange of defective media
 * within 90 days of receipt.
 *
 * Limitations on Rights to Redistribute This Code
 *
 * Unicode, Inc. hereby grants the right to freely use the information
 * supplied in this file in the creation of products supporting the
 * Unicode Standard, and to make copies of this file in any form
 * for internal or external distribution as long as this notice
 * remains attached.
 */

/* ---------------------------------------------------------------------

    Conversion from UTF-8 to UTF-16.  Header file.

	Stripped and modified for John the Ripper ; see ConvertUTF.h.original
	for the original content. magnum, 2009

    Author: Mark E. Davis, 1994.
    Rev History: Rick McGowan, fixes & updates May 2001.
		 Fixes & updates, Sept 2001.

------------------------------------------------------------------------ */

/* ---------------------------------------------------------------------
    The following definitions are compiler-specific.
    The C standard does not guarantee that wchar_t has at least
    16 bits, so wchar_t is no less portable than unsigned short!
    All should be unsigned values to avoid sign extension during
    bit mask & shift operations.
------------------------------------------------------------------------ */

#ifndef _CONVERTUTF_H
#define _CONVERTUTF_H

typedef unsigned long UTF32;	/* at least 32 bits */
typedef unsigned short UTF16;	/* at least 16 bits */
typedef unsigned char UTF8;	/* typically 8 bits */

/* Some fundamental constants */
#define UNI_REPLACEMENT_CHAR (UTF32)0x0000FFFD
#define UNI_MAX_BMP (UTF32)0x0000FFFF
#define UNI_MAX_UTF16 (UTF32)0x0010FFFF
#define UNI_MAX_UTF32 (UTF32)0x7FFFFFFF
#define UNI_MAX_LEGAL_UTF32 (UTF32)0x0010FFFF

/* These are used in NT_fmt.c */
extern const UTF32 offsetsFromUTF8[];
extern const char opt_trailingBytesUTF8[64];

/* Convert UTF-8 to UTF-16LE */
extern int utf8towcs(UTF16 * target, unsigned int maxtargetlen,
    const UTF8 * source, unsigned int sourcelen);

/* Convert to UTF-16LE from UTF-8 or ISO-8859-1 depending on --utf8 flag */
extern int plaintowcs(UTF16 * dst, unsigned int maxdstlen, const UTF8 * src,
    unsigned int srclen);

/* Convert back to UTF-8 (for get_key without a saved_plain) */
extern char * utf16toutf8 (const UTF16* source);

/* These were in smbencrypt.c before: */

/* Return length (in characters) of a UTF16 string */
/* Number of octets is the result * sizeof(UTF16)  */
extern unsigned int strlen16(const UTF16 * str);

/* Create an NT hash from a ISO-8859 or UTF-8 string (--utf8 aware) */
extern int E_md4hash(const UTF8 * passwd, unsigned int len, unsigned char *p16);

/* --------------------------------------------------------------------- */
#endif				/* _CONVERTUTF_H */
